#include "input.h"
#include "buzzer.h"
#include "screen.h"
#include "characters.h"
#include "number_input.h"

String numberInput = "";
int8_t dotPosition = -1;

#define MAX_LENGTH 8

void number_input_update();

float number_input(float defaultValue, float encoderStep) {
  // setup
  if (defaultValue < 0) {
    numberInput = "";
    dotPosition = -1;
  } else {
    // see how many decimal places the default value has (max 3)
    unsigned char decimalPlaces = 3;
    numberInput = String(defaultValue, decimalPlaces);
    if (numberInput.endsWith(".000")) {
      decimalPlaces = 0;
    } else if (numberInput.endsWith("00")) {
      decimalPlaces = 1;
    } else if (numberInput.endsWith("0")) {
      decimalPlaces = 2;
    }

    // stringify again
    numberInput = String(defaultValue, decimalPlaces);
    dotPosition = numberInput.indexOf('.');

    if (numberInput.length() > MAX_LENGTH) {
      beep();
      numberInput = numberInput.substring(0, MAX_LENGTH);
    }
  }

  // set up screen
  lcd.cursor();
  number_input_update();

  // loop
  while (true) {
    auto input = handleInput();

    auto len = numberInput.length();
    bool isAtLength = len >= MAX_LENGTH;

    if (input.key == KEY_C || input.switchEvent == SWITCH_RELEASED) {
      if (numberInput.length() > 0) {
        lcd.noCursor();
        return numberInput.toFloat();
      } else {
        beep();
      }
    } else if (input.key) {
      auto digit = getPressedDigit(input.key);
      if (digit != -1) {
        // append digit (if it fits)
        if (isAtLength) {
          beep();
        } else {
          numberInput += digit;
          tick();
        }
      } else if (input.key == KEY_ASTERISK) {
        // append decimal point (if possible)
        if (isAtLength) {
          beep();
        } else if (dotPosition == -1 && len >= 1) {
          dotPosition = numberInput.length();
          numberInput += ".";
          tick();
        } else {
          beep();
        }
      } else if (input.key == KEY_HASH) {
        // backspace
        if (len) {
          if (dotPosition == len - 1) {
            dotPosition = -1;
          }
          numberInput = numberInput.substring(0, len - 1);
          tick();
        } else {
          beep();
        }
      } else if (input.key == KEY_B) {
        lcd.noCursor();
        return NUMBER_INPUT_BACK;
      } else {
        beep();
      }

      number_input_update();
    } else if (input.encoderEvent) {
      // normalize input
      if (numberInput.length() == 0) {
        numberInput = "0";
      }

      // remember decimal places
      len = numberInput.length();
      auto previousDecimalPlaces = dotPosition == -1 ? 0 : len - dotPosition - 1;

      // calculate new value
      auto wireDia = numberInput.toFloat();
      wireDia += input.encoderEvent * encoderStep;

      // clamp to 0
      if (wireDia < 0) {
        wireDia = 0;
        beep();
      } else {
        tick();
      }

      // stringify
      numberInput = String(wireDia, max(previousDecimalPlaces, 1));
      dotPosition = numberInput.indexOf('.');

      number_input_update();
    } else if (input.switchEvent == SWITCH_RELEASED) {
    } else if (input.hasInput && input.switchEvent != SWITCH_PRESSED) {
      beep();
    }
  }
}


void number_input_update() {
  lcd.setCursor(0, 1);
  lcd.print(numberInput);
  // Delete following character (if backspace was pressed), then move cursor back to position
  lcd.write(' ');
  lcd.setCursor(numberInput.length(), 1);
}
